function [l, s, Vm, Vs, Tm, Ts] = holdoutCVKernLR(X, Y, kernel, perc, nrip, intLambda, intKerPar)
%[l, s, Vm, Vs, Tm, Ts] = holdoutCVKernLR(X, Y, kernel, perc, nrip, intLambda, intKerPar)
% Xtr: the training examples
% Ytr: the training labels
% kernel: the kernel function (see help KernelMatrix).
% perc: percentage of the dataset to be used for validation
% nrip: number of repetitions of the test for each couple of parameters
% intLambda: list of regularization parameters 
%       for example intLambda = [5,2,1,0.7,0.5,0.3,0.2,0.1, 0.05, 0.02, 0.01, 0.005, 0.002, 0.001, 0.0005, 0.0002, 0.0001,0.00001,0.000001];
% intKerPar: list of kernel parameters 
%       for example intKerPar = [10,7,5,4,3,2.5,2.0,1.5,1.0,0.7,0.5,0.3,0.2,0.1, 0.05, 0.03,0.02, 0.01];
% 
% Output:
% l, s: the couple of lambda and kernel parameter that minimize the median of the
%       validation error
% Vm, Vs: median and variance of the validation error for each couple of parameters
% Tm, Ts: median and variance of the error computed on the training set for each couple
%       of parameters
%
% Example of code:
% intLambda = [5,2,1,0.7,0.5,0.3,0.2,0.1, 0.05, 0.02, 0.01, 0.005, 0.002, 0.001, 0.0005, 0.0002, 0.0001,0.00001,0.000001];
% intKerPar = [10,7,5,4,3,2.5,2.0,1.5,1.0,0.7,0.5,0.3,0.2,0.1, 0.05, 0.03,0.02, 0.01];
% [Xtr, Ytr] = MixGauss([[0;0],[1;1]],[0.5,0.25],100);
% [l, s, Vm, Vs, Tm, Ts] = holdoutCVKernLR(Xtr, Ytr,'gaussian', 0.5, 5, intLambda, intKerPar);
% figure
% h = surf(intKerPar , intLambda, Vm);
% set(h,'FaceColor',[0 0 1])   
% alpha(h,0.4)
% hold on
% h = surf(intKerPar , intLambda,  Tm);
% set(h,'FaceColor',[0 1 0])   
% alpha(h,0.4)
% set(gca,'xscale','log');
% set(gca,'yscale','log');
% xlabel('\sigma')
% ylabel('\lambda')
% zlabel('Error')
% legend('Validation','Training')

    nKerPar = numel(intKerPar);
    nLambda = numel(intLambda);
 
    
    n = size(X,1);
    ntr = ceil(n*(1-perc));
    
    tmn = zeros(nLambda, nKerPar, nrip);
    vmn = zeros(nLambda, nKerPar, nrip);
    
    for rip = 1:nrip
        I = randperm(n);
        Xtr = X(I(1:ntr),:);
        Ytr = Y(I(1:ntr),:);
        Xvl = X(I(ntr+1:end),:);
        Yvl = Y(I(ntr+1:end),:);
        
        
        is = 0;
        for s=intKerPar
            is = is + 1;
            il = 0;  
            
            c = zeros(ntr,1);
            for l=intLambda
                il = il + 1;
         
                
                c = kernLRTrain1(Xtr, Ytr, kernel, s, l, c);
                
                tmn(il, is, rip) =  calcErr(kernLRTest(c, Xtr, kernel, s, Xtr), Ytr);               
                vmn(il, is, rip)  = calcErr(kernLRTest(c, Xtr, kernel, s, Xvl), Yvl);
                
                str = sprintf('l\ts\tvalErr\ttrErr\n%f\t%f\t%f\t%f\n', l, s, vmn(il,is,rip), tmn(il,is,rip));
                disp(str);
            end
        end
    end
    
    Tm = median(tmn,3);
    Ts = std(tmn,0,3);
    Vm = median(vmn,3);
    Vs = std(vmn,0,3);
    
    [row, col] = find(Vm <= min(min(Vm)));
    
    l = intLambda(row(1));
    s = intKerPar(col(1));
end

function err = calcErr(T, Y)
    %vY = (Y + 1)/2;
    %err = sum(abs(vY-T))/numel(Y);
    %err = sum(Y ~= sign(T-0.5))/numel(Y)
    err = sum(log(1+exp(-Y.*T)))/numel(Y);
end

function c = kernLRTrain1(Xtr, Ytr, kernel, kerpar, lambda, c)
    iter = 10000;
    epsilon = min(1e-6, 0.1*lambda);
    [n, d] = size(Xtr);
    K = KernelMatrix(Xtr, Xtr, kernel, kerpar);
    L = eigs(K, 1)/n + lambda;
    gamma = 1/L;
    j = 0;
    fold = 0;
    f = inf;
    while(j < iter && abs(f - fold) >= epsilon)
        fold = f;
        j = j + 1;
        p = exp(Ytr.*(K*c));
        c = c - gamma*(-(1/n)*(Ytr./(1+p)) + 2*lambda*c);
        f = sum(log(1 + exp(-Ytr.*(K*c))))/n + lambda*c'*c;
        %disp(['iter:', num2str(j),'  err:', num2str(abs(f - fold))]);
    end
end






